package com.flyfish.framework.compiler.support;

import com.flyfish.framework.compiler.DynamicJavaCompiler;
import com.flyfish.framework.compiler.core.ClassLoaders;
import com.flyfish.framework.compiler.core.JavaStringCompiler;
import org.apache.commons.lang3.StringUtils;

import java.util.Map;

/**
 * 使用第三方成熟框架
 *
 * @author wangyu
 */
public class DelegateJavaCompiler implements DynamicJavaCompiler {

    // 第三方编译器
    private final JavaStringCompiler compiler = new JavaStringCompiler(ClassLoaders.memory());

    private DelegateJavaCompiler() {
    }

    public static DelegateJavaCompiler getInstance() {
        return SingletonHolder.INSTANCE;
    }

    /**
     * 编译java源码，通过字符串
     *
     * @param name   文件名
     * @param source 源码
     * @return 结果
     */
    @Override
    public Class<?> compile(String name, String source) throws ClassNotFoundException {
        Map<String, byte[]> byteMap = compiler.compile(name, source);
        String className = StringUtils.substringBeforeLast(name, ".");
        for (String key : byteMap.keySet()) {
            if (key.contains(className)) {
                return compiler.loadClass(key);
            }
        }
        throw new RuntimeException("【java编译器】源码中没有可以编译的类！");
    }

    @Override
    public void close() {
        compiler.close();
    }

    private static class SingletonHolder {

        private static final DelegateJavaCompiler INSTANCE = new DelegateJavaCompiler();
    }


//    public static void main(String[] args) throws ClassNotFoundException, IOException {
//        long time = System.currentTimeMillis();
//        System.out.println(StreamUtils.copyToString(DelegateJavaCompiler.class.getResourceAsStream("./"), StandardCharsets.UTF_8));
//        Class<?> clazz = getInstance().compile("Fuck.java", "package com.flyfish.project; public class Fuck {}");
//        System.out.println(clazz + "耗时:" + (System.currentTimeMillis() - time) / 1000.0 + "秒");
//    }
}
