package com.flyfish.framework.utils;

import com.flyfish.framework.domain.base.Domain;

import java.util.ArrayList;
import java.util.List;
import java.util.function.BiFunction;
import java.util.stream.IntStream;

/**
 * 数据存储使用工具
 *
 * @author wybab
 */
public final class DataUtils {

    /**
     * 自增
     *
     * @param count 数量
     * @return 结果
     */
    public static int increment(Integer count) {
        if (null == count) {
            return 1;
        }
        return count + 1;
    }

    /**
     * 创建ref对象
     *
     * @param clazz 类
     * @param id    主键
     * @param <T>   泛型
     * @return 结果
     */
    @SuppressWarnings("deprecation")
    public static <T extends Domain> T ref(Class<T> clazz, String id) {
        try {
            T instance = clazz.newInstance();
            instance.setId(id);
            return instance;
        } catch (InstantiationException | IllegalAccessException e) {
            e.printStackTrace();
        }
        return null;
    }

    /**
     * 合并对象
     * @param keys 键们
     * @param names 名称们
     * @param mapper 映射
     * @param <T> 泛型参数
     * @param <R> 泛型结果
     * @return 结果
     */
    public static <T, R> List<R> zip(String[] keys, T[] names, BiFunction<String, T, R> mapper) {
        return IntStream.range(0, Math.min(keys.length, names.length))
                .boxed()
                .reduce(new ArrayList<>(), (result, index) -> {
                    R mapped = mapper.apply(keys[index], names[index]);
                    if (null != mapped) {
                        result.add(mapped);
                    }
                    return result;
                }, (a, b) -> a);
    }

    /**
     * 创建ref对象
     *
     * @param instance 实例
     * @param <T>      泛型
     * @return 结果
     */
    @SuppressWarnings("unchecked")
    public static <T extends Domain> T ref(T instance) {
        Class<T> tClass = (Class<T>) instance.getClass();
        return ref(tClass, instance.getId());
    }
}
