package com.flyfish.framework.service;

import com.flyfish.framework.domain.AdminUserDetails;
import com.flyfish.framework.domain.DepartmentQo;
import com.flyfish.framework.domain.authorized.AuthorizedUserDetails;
import com.flyfish.framework.domain.base.Qo;
import com.flyfish.framework.domain.po.Department;
import com.flyfish.framework.enums.UserType;
import com.flyfish.framework.service.impl.BaseReactiveServiceImpl;
import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.lang3.BooleanUtils;
import org.springframework.data.mongodb.core.ReactiveMongoOperations;
import org.springframework.data.mongodb.core.query.Criteria;
import org.springframework.data.mongodb.core.query.Query;
import org.springframework.stereotype.Service;
import reactor.core.publisher.Flux;
import reactor.core.publisher.Mono;

import javax.annotation.Resource;
import java.util.List;
import java.util.Set;
import java.util.stream.Collectors;

/**
 * 部门服务
 *
 * @author wangyu
 */
@Service
public class DepartmentService extends BaseReactiveServiceImpl<Department> {

    @Resource
    private ReactiveMongoOperations reactiveMongoOperations;

    /**
     * 获取子部门列表
     *
     * @return 结果
     */
    public Mono<Set<String>> getSubCodes(Set<String> parents) {
        Query query = Query.query(Criteria.where("parentIds").in(parents));
        query.fields().include("_id");
        return reactiveMongoOperations.find(query, Department.class).map(Department::getId).collect(Collectors.toSet());
    }

    /**
     * 查询列表
     *
     * @param query 查询
     * @return 结果
     */
    @Override
    public Flux<Department> getList(Qo<Department> query) {
        if (query instanceof DepartmentQo) {
            DepartmentQo qo = (DepartmentQo) query;
            // 如果是非管理员
            if (qo.getUser().getType() != UserType.SUPER_ADMIN) {
                // 使用列表内的部门作为条件
                AdminUserDetails userDetails = (AdminUserDetails) query.getUser();
                // 查询根节点下的节点
                if (Department.ROOT.equals(qo.getParentId())) {
                    qo.setParentId(null);
                    qo.setIds(userDetails.getVisibleDeparts());
                    // 不递归，指定深度，保证单层
                    if (BooleanUtils.isNotTrue(qo.getRecursive())) {
                        // 指定深度
                        int maxDepth = userDetails.getDepartments().stream().max((a, b) -> a.getDepth() - b.getDepth())
                                .map(Department::getDepth)
                                .orElse(0);
                        qo.setDepth(maxDepth);
                    }
                }
                if (null == qo.getParentId() && CollectionUtils.isEmpty(qo.getParentIds())) {
                    qo.setIds(userDetails.getVisibleDeparts());
                }
            }
        }
        return super.getList(query);
    }
}
