package com.flyfish.framework.handler;

import com.flyfish.framework.bean.Result;
import com.flyfish.framework.service.AuthenticationAuditor;
import lombok.RequiredArgsConstructor;
import org.springframework.http.HttpHeaders;
import org.springframework.http.MediaType;
import org.springframework.http.server.reactive.ServerHttpResponse;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.userdetails.UserDetails;
import org.springframework.security.web.server.WebFilterExchange;
import org.springframework.security.web.server.authentication.ServerAuthenticationSuccessHandler;
import reactor.core.publisher.Mono;

/**
 * 登录成功的回调
 *
 * @author wangyu
 */
@RequiredArgsConstructor
public class JsonAuthenticationSuccessHandler implements ServerAuthenticationSuccessHandler {

    // 数据块工厂
    private final AuthenticationAuditor authenticationAuditor;

    /**
     * 登录成功后要返回用户的基本信息，节省带宽
     *
     * @param webFilterExchange 请求信息
     * @param authentication    认证信息
     * @return 结果
     */
    @Override
    public Mono<Void> onAuthenticationSuccess(WebFilterExchange webFilterExchange, Authentication authentication) {
        ServerHttpResponse response = webFilterExchange.getExchange().getResponse();
        HttpHeaders headers = response.getHeaders();
        headers.setContentType(MediaType.APPLICATION_JSON);
        Object principal = authentication.getPrincipal();
        // 如果登录成功，清空错误次数，更新上次登录时间
        return authenticationAuditor.success((UserDetails) principal)
                .then(response.writeWith(authenticationAuditor.transform(Result.accept(principal))));
    }
}
